#!/bin/sh
#
# Script to update the MBR with correct information after a kernel 
# installation / upgrade
#
# Who wrote this hack?
#
# v0.1.2    2003-06-16, christht
# v0.1.1    2003-06-13, christht
# v0.1.0    2003-06-03, christht
# v0.0.9    2003-05-20, christht
# v0.0.8    2003-05-20, christht
# v0.0.7    2003-05-20, christht
# v0.0.6    2003-05-02, christht
# v0.0.5    2003-04-24, christht
# v0.0.4    2003-04-23, christht
# v0.0.3    2003-04-23, christht
# v0.0.2    2003-04-22, christht
# v0.0.1    2003-04-07, christht
#
# This script takes one argument: kernelversion
# Ex: 2.4.10-23trsmp
#
# It reads configuration variables from the file:
#   /etc/sysconfig/bootloader
# or whatever is defined by $CONFIG
#
# The following variables are used:
#   BOOTLOADER=[grub,lilo,none]
#   AUTOACTIVATE=[yes,no]
#   REBOOT=[yes,no]
#   REBOOTAT=(see shutdown manpage for syntax)
#   MAILTO=
#
# Possible cases when upgrading kernel:
#
#
# 0: We are upgrading on a newly installed system; current symlink and old
#    symlink points to the same kernel / initrd
#
#
# 1: New kernel has same type as booted
#    1.1: Booted kernel is linked by initrd[-type].img
#        Remove initrd[-type].img.old
#        Rename initrd[-type].img initrd[-type].img.old
#    1.2: Booted kernel is linked by initrd[-type].img.old
#        Remove initrd[-type].img
#    1.3: Booted kernel is not linked with any of above.
#        Remove initrd[-type].img.old
#        Rename initrd[-type].img initrd[-type].img.old
#
# 2: New kernel has other type than booted
#    Remove initrd[-type].img.old
#    Rename initrd[-type].img initrd[-type].img.old
#
# In any of the above cases:
#   create initrd[-type].img
#   link initrd[-type]-kver.img initrd[-type].img

NRARGS=$#

# check number of arguments
if [ ! "$NRARGS" = "1" ]; then
    echo "Wrong number of arguments, exiting!"
    echo "Usage:"
    echo "$0 <kernel-version-release>"
    exit 1
fi

#
# A file to add content of possible mail:
#
MAILFILE=$(mktemp /tmp/kernmail.XXXXXX)


#
# Some functions:
#

function mailto(){
if [ "$MAILTO" != "" ]; then
    if [ -f "$MAILFILE" ]; then
	if [ -x /bin/hostname ]; then
	    HOSTNAME=$(/bin/hostname)
	    SUBJ="Kernel upgrade on '$HOSTNAME'"
	else
	    SUBJ='Kernel upgrade'
	fi
	    cat $MAILFILE | mail -s "$SUBJ" $MAILTO > /dev/null 2>&1
    else
	echo "Could not find mail content: $MAILFILE"
    fi
fi
rm -f $MAILFILE >/dev/null 2>&1 || :
}

function abandon(){
mailto
exit 1
}

#
# Setting default variables:
# ( Paranoid, do nothing! )
# 
CONFIG='/etc/sysconfig/bootloader'
BOOTLOADER=none
AUTOACTIVATE=no
REBOOT=no
REBOOTAT=+60
MAILTO=

#
# Get values from configuration file:
#
if [ -f "$CONFIG" ]; then
    . $CONFIG
else
    echo "File '$CONFIG' not found, using defaults." \
	| tee -a $MAILFILE
fi
    
KVER=$1
DIR='/boot'
IMG="initrd-$KVER.img"
VML="vmlinuz-$KVER"

# 
# create initrd, using arguments.
# If this turns out broken, break all.
#
pushd $DIR >/dev/null 2>&1
mkinitrd -f $IMG $KVER > /dev/null 2>&1 ||\
	( echo "Errors when creating initrd!"\
	| tee -a $MAILFILE \
	 && abandon )
popd > /dev/null 2>&1

#
# Simple test to see if initrd _looks_ ok:
#
pushd $DIR >/dev/null 2>&1
OK=$(file $IMG | grep -q gzip; echo $?)
if [ ! "$OK" = "0" ]; then
    echo "initrd does not look valid. Exiting!" \
	| tee -a $MAILFILE	
    abandon
fi
popd >/dev/null 2>&1

#
# Check if we want to activate the new kernel:
#
if [ "$AUTOACTIVATE" != "yes" ]; then
    echo "Not configured to autoactivate new kernel." \
	| tee -a $MAILFILE
    mailto
    exit 0
fi

#
#   check _type_ of running kernel:
#
CURRENT=$(uname -r)
KUP=`echo $CURRENT | grep -q \\\-.*[0-9]*[a-z][a-z]$ ; echo $?`
KSMP=`echo $CURRENT | grep -q \\\-.*[0-9]*[a-z][a-z]smp$ ; echo $?`
FWUP=`echo $CURRENT | grep -q \\\-.*[0-9]*[a-z][a-z]firewall$ ; echo $?`
FWSMP=`echo $CURRENT | grep -q \\\-.*[0-9]*[a-z][a-z]firewallsmp$ ; echo $?`

#
#   check type of new kernel:
#
NEW_KUP=`echo $KVER | grep -q \\\-.*[0-9]*[a-z][a-z]$ ; echo $?`
NEW_KSMP=`echo $KVER | grep -q \\\-.*[0-9]*[a-z][a-z]smp$ ; echo $?`
NEW_FWUP=`echo $KVER | grep -q \\\-.*[0-9]*[a-z][a-z]firewall$ ; echo $?`
NEW_FWSMP=`echo $KVER | grep -q \\\-.*[0-9]*[a-z][a-z]firewallsmp$ ; echo $?`

#
# Decide which symlink to use:
#
SYMIMG="NULL"
SYMVML="NULL"
if [ "$FWSMP" = "0" -a "$NEW_FWSMP" = "0" ]; then
    SYMIMG='initrd-firewall-smp.img'
    SYMVML='vmlinuz-firewall-smp'
elif [ "$FWUP" = "0" -a "$NEW_FWUP" = "0" ]; then
    SYMIMG='initrd-firewall.img'
    SYMVML='vmlinuz-firewall'
elif [ "$KSMP" = "0" -a "$NEW_KSMP" = "0" ]; then
    SYMIMG='initrd-smp.img'
    SYMVML='vmlinuz-smp'
elif [ "$KUP" = "0" -a "$NEW_KUP" = "0" ]; then
    SYMIMG='initrd.img'
    SYMVML='vmlinuz'
fi

#
# Update symlinks corresponding to the current case:
#

pushd $DIR >/dev/null 2>&1
# We like to have all expected symlinks present:
  
if [ "$SYMIMG" != "NULL" ]; then
    NEW=$(file $SYMIMG | grep -q $CURRENT)
    OLD=$(file $SYMIMG.old | grep -q $CURRENT)
    if [ "$NEW" = "0" ]; then
        rm -f $SYMIMG.old || :
        rm -f $SYMVML.old || :
        mv $SYMIMG $SYMIMG.old || :
        mv $SYMVML $SYMVML.old || :
        ln -s $IMG $SYMIMG || :
        ln -s $VML $SYMVML || :
    elif [ "$OLD" = "0" ]; then
        rm -f $SYMIMG || :
        rm -f $SYMIMG || :
        ln -s $IMG $SYMIMG || :
        ln -s $VML $SYMVML || :
  	fi
else
    # case 2:
    if [ "$NEW_FWSMP" = "0" ]; then
        SYMIMG='initrd-firewall-smp.img'
        SYMVML='vmlinuz-firewall-smp'
    elif [ "$NEW_FWUP" = "0" ]; then
        SYMIMG='initrd-firewall.img'
        SYMVML='vmlinuz-firewall'
    elif [ "$NEW_KSMP" = "0" ]; then
        SYMIMG='initrd-smp.img'
        SYMVML='vmlinuz-smp'
    elif [ "$NEW_KUP" = "0" ]; then
        SYMIMG='initrd.img'
        SYMVML='vmlinuz'
    fi
fi

if [ "$SYMIMG" != NULL ]; then
    rm -f $SYMIMG.old || :
    rm -f $SYMVML.old || :
    mv $SYMIMG $SYMIMG.old || :
    mv $SYMVML $SYMVML.old || :
    ln -s $IMG $SYMIMG || :
    ln -s $VML $SYMVML || :
else
    echo "The kernel version '$KVER' does not seem to be supported."\
	 | tee -a $MAILFILE
    abandon
fi
popd > /dev/null 2>&1
  
#
# We may want to update MBR since AUTOACTIVATE='yes'
#
if [ "$BOOTLOADER" = "lilo" -a -x /sbin/lilo ]; then
      /sbin/lilo 2>&1 > /dev/null
elif [ "$BOOTLOADER" != "grub" ]; then
      echo "No valid bootloader set. Not updating MBR." \
	| tee -a $MAILFILE
fi
  
#
# Are we going to reboot?
#
if [ "$REBOOT" = "yes" -a ! -z "$REBOOTAT" ]; then
    # shutdown only if $REBOOTAT != [now,0] inn background
    if [ "$REBOOTAT" != "now" -a "$REBOOTAT" != "+0" ]; then
        shutdown -r $REBOOTAT "Rebooting due to kernel upgrade."  & \
	    > /dev/null 2>&1
        echo "" | tee -a $MAILFILE
        echo "Rebooting due to kernel upgrade. " \
	    | tee -a $MAILFILE
	echo "New kernel version is: '$KVER'" \
	    | tee -a $MAILFILE
        echo "" | tee -a $MAILFILE
	echo "The time argument given to shutdown was: '$REBOOTAT'" \
	    | tee -a $MAILFILE
        echo "Use 'shutdown -c' to cancel shutdown." \
	    | tee -a $MAILFILE
    else
        echo "To reboot _now_ (during upgrade) is probably not a good idea.."\
	    | tee -a $MAILFILE
        echo "Please re-evaluate your 'REBOOTAT' setting."\
	    | tee -a $MAILFILE
    fi
else
    echo "Will not reboot." \
	| tee -a $MAILFILE
fi


#
# Done
#
mailto
exit 0
